@@ -2,12 +2,11 @@
 
 case "${1}" in
     install)
-        git clone --depth=1 https://github.com/certbot/certbot
         cd certbot
-        git rev-parse HEAD
-        tools/pip_install_editable.py ./acme[test]
-        tools/pip_install_editable.py ./certbot[test]
-        pip install -U pyopenssl
+        uv pip install pip
+        tools/pip_install.py -e ./acme[test]
+        tools/pip_install.py -e ./certbot[test]
+        uv pip install -U pyopenssl
         ;;
     run)
         cd certbot
