@@ -2,11 +2,9 @@
 
 case "${1}" in
     install)
-        git clone --depth=1 https://github.com/awslabs/aws-dynamodb-encryption-python
         cd aws-dynamodb-encryption-python
-        git rev-parse HEAD
-        pip install -e .
-        pip install -r test/upstream-requirements-py311.txt
+        uv pip install -e .
+        uv pip install -r test/upstream-requirements-py311.txt
         ;;
     run)
         cd aws-dynamodb-encryption-python
