@@ -2,11 +2,8 @@
 
 case "${1}" in
     install)
-        pip install uv
-        git clone --depth=1 https://github.com/mitmproxy/mitmproxy
         cd mitmproxy
-        git rev-parse HEAD
-        uv pip install --system --group dev -e .
+        uv pip install -r <(uv export --locked) -e .
         ;;
     run)
         cd mitmproxy
