@@ -8,7 +8,7 @@ use std::str::FromStr;
 use asn1::IA5String;
 
 // RFC 2822 3.2.4
-static ATEXT_CHARS: &str = "!#$%&'*+-/=?^_`{|}~";
+const ATEXT_CHARS: &str = "!#$%&'*+-/=?^_`{|}~";
 
 /// Represents a DNS name can be used in X.509 name matching.
 ///
@@ -138,6 +138,19 @@ impl<'a> DNSPattern<'a> {
             },
         }
     }
+
+    /// Returns the inner `DNSName` within this `DNSPattern`, e.g.
+    /// `foo.com` for `*.foo.com` or `example.com` for `example.com`.
+    ///
+    /// This API must not be used to bypass pattern matching; it exists
+    /// solely to enable checks that only require the inner name, such
+    /// as Name Constraint checks.
+    pub fn inner_name(&self) -> &DNSName<'a> {
+        match self {
+            DNSPattern::Exact(dnsname) => dnsname,
+            DNSPattern::Wildcard(dnsname) => dnsname,
+        }
+    }
 }
 
 /// A `DNSConstraint` represents a DNS name constraint as defined in [RFC 5280 4.2.1.10].
